/*
 * $Id: BulletImpl.java,v 1.1.1.1 2002/07/07 10:27:57 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.bulletsmorph;

import jp.gr.java_conf.abagames.bulletml.*;
import jp.gr.java_conf.abagames.util.*;

/**
 * Bullet implementation.
 *
 * @version $Revision: 1.1.1.1 $
 */
public class BulletImpl {
  public static final int NOT_EXIST = Integer.MIN_VALUE;

  private final int ACTION_MAX = 8;

  private ActionImpl[] action = new ActionImpl[ACTION_MAX];
  private int acIdx;

  public int x, y, px, py, width;

  public Direction drcElm;
  public Speed spdElm;
  public float direction, speed;

  public float mx, my;

  public int clr;

  private int cnt;

  private BulletmlManager manager;

  private float[] prms;

  private int tmx, tmy;
  private BulletImpl parent;
  public float rank;

  private Bullet bulletElm;

  public BulletImpl(BulletmlManager manager) {
    this.manager = manager;
    x = NOT_EXIST;
  }

  public void changeAction(ActionImpl bfr, ActionImpl aft) {
    for ( int i=0 ; i<acIdx ; i++ ) {
      if ( action[i].equals(bfr) ) {
        action[i] = aft;
        return;
      }
    }
  }

  public void set(IActionElmChoice[] aec,
   int x, int y, int d, int clr, float rank, boolean isTop) {
    if ( bulletElm == null ) {
      bulletElm = new Bullet();
      bulletElm.setActionElm(aec);
    }
    this.x = px = x; this.y = py = y;
    mx = my = 0; tmx = tmy = 0;
    this.clr = clr;
    cnt = 0;
    acIdx = 0;

    String fileName = manager.getFileName();

    if ( manager.bulletmlUtil.addActionToNormBullet && aec.length == 0 ) {
      aec = new IActionElmChoice[1];
      Action act = new Action();
      act.setContent(new IActionChoice[0]);
      aec[0] = act;
      bulletElm.setActionElm(aec);
    }
    for ( int i=0 ; i<aec.length ; i++ ) {
      action[acIdx] = manager.getActionImplInstance();
      if ( action[acIdx] == null ) break;
      action[acIdx].set(manager.bulletmlUtil.getActionElm(aec[i], fileName), this);
      float[] actPrms = manager.bulletmlUtil.getActionParams(aec[i], prms);
      if ( actPrms == null ) {
        action[acIdx].setParams(prms);
      } else {
        action[acIdx].setParams(actPrms);
      }

      manager.bulletmlUtil.addActionImpl(action[acIdx], isTop);

      acIdx++;
      if ( acIdx >= ACTION_MAX ) break;
    }
    direction = d;
    speed = 0;
    this.rank = rank;
    parent = null;
  }

  public void set(Bullet bullet, int x, int y, int ci, BulletImpl bImpl) {
    bulletElm = bullet;
    drcElm = bullet.getDirection();
    spdElm = bullet.getSpeed();
    IActionElmChoice[] aec = bullet.getActionElm();
    set(aec, x, y, 0, ci, bImpl.rank, false);
    parent = bImpl;
  }

  public void setParams(float[] prms) {
    this.prms = prms;
  }

  public float getAimDeg() {
    return (float)DegUtil.getDeg(manager.shipx - x, manager.shipy - y)
     * 360 / SCTable.TABLE_SIZE;
  }

  public void vanish() {
    for ( int i=0 ; i<acIdx ; i++ ) {
      action[i].vanish();
    }
    x = NOT_EXIST;
  }

  public final boolean isAllActionFinished() {
    for ( int i=0 ; i<acIdx ; i++ ) {
      if ( !action[i].isFinished() ) {
        return false;
      }
    }
    return true;
  }

  public final void move() {
    manager.bulletmlUtil.setRank(rank);
    try {
      for ( int i=0 ; i<acIdx ; i++ ) {
        action[i].move();
      }
      cnt++;
    } catch ( Exception e ) {
      e.printStackTrace();
      vanish();
      return;
    }

    int d = (int)(direction*SCTable.TABLE_SIZE/360);
    d &= (SCTable.TABLE_SIZE-1);

    int mvx = ((int)(speed*SCTable.sintbl[d])<<1) + (int)(mx*512);
    int mvy = ((int)(-speed*SCTable.costbl[d])<<1) + (int)(my*512);
    int pmvx = mvx, pmvy = mvy;
    x += mvx; tmx = mvx;
    y += mvy; tmy = mvy;
    if ( pmvx == 0 && pmvy == 0 ) {
      pmvx = SCTable.sintbl[d];
      pmvy = -SCTable.costbl[d];
    }
    if ( cnt < 4 ) {
      px = x - pmvx;
      py = y - pmvy;
    } else if ( cnt < 8 ) {
      px = x - (pmvx<<1);
      py = y - (pmvy<<1);
    } else {
      px = x - (pmvx<<2);
      py = y - (pmvy<<2);
    }

    if ( isAllActionFinished() ) {
      if ( px < 0 || px >= Screen.WIDTH<<8 ||
           py < 0 || py >= Screen.HEIGHT<<8 ) {
        vanish();
      }
    } else {
      if ( px < -(Screen.WIDTH<<6) ||
           px >= (Screen.WIDTH<<8)+(Screen.WIDTH<<6) ||
           py < -(Screen.HEIGHT<<6) ||
           py >= (Screen.HEIGHT<<8)/*+(Screen.HEIGHT<<6)*/ ) {
        vanish();
      }
    }
    if ( isAllActionFinished() && speed < 0.2f ) {
      vanish();
    }
  }

  public final void draw() {
    manager.drawBullet(x, y, px, py, clr);
  }
}
