/*
 * $Id: BulletmlManager.java,v 1.1.1.1 2002/07/07 10:27:57 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.bulletsmorph;

import jp.gr.java_conf.abagames.bulletml.*;
import jp.gr.java_conf.abagames.util.*;
import java.io.*;
import java.util.Vector;
import java.util.Random;
import java.awt.TextArea;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;

/**
 * Handle bulletimpls and actionimpls.
 *
 * @version $Revision: 1.1.1.1 $
 */
public class BulletmlManager {
  private BulletmlPlayer player;

  private final int BULLET_MAX = 256;
  private BulletImpl[] bullet = new BulletImpl[BULLET_MAX];
  private int bltIdx = 0;

  private final int ACTION_MAX = 1024;
  private ActionImpl[] action = new ActionImpl[ACTION_MAX];
  private int actIdx = 0;

  public int shipx, shipy;

  private String fileName;

  private int cnt;
  private final int CNT_MAX = 60*10;

  private Vector actionsCatalog;

  public BulletmlUtil bulletmlUtil = new BulletmlUtil();

  private GPEngine engine;

  public void init() {
    for ( int i=0 ; i<bullet.length ; i++ ) {
      bullet[i] = new BulletImpl(this);
    }
    for ( int i=0 ; i<action.length ; i++ ) {
      action[i] = new ActionImpl(this);
    }
    cnt = 0;
  }

  public void initBulletmlUtil() {
    bulletmlUtil.clear();
  }

  public void setPlayer(BulletmlPlayer player) {
    this.player = player;
  }

  public void setGPEngine(GPEngine engine) {
    this.engine = engine;
  }

  public String getFileName() {
    return fileName;
  }

  public void setFileName(String fileName) {
    this.fileName = fileName;
  }

  public void setShipPos(int x, int y) {
    shipx = x; shipy = y;
  }

  public void setActionsCatalog(Vector ac) {
    actionsCatalog = ac;
  }

  public Vector getActionsCatalog() {
    return actionsCatalog;
  }

  public BulletImpl getBulletImplInstance() {
    for ( int i=BULLET_MAX-1 ; i>=0 ; i-- ) {
      bltIdx++; bltIdx &= (BULLET_MAX-1);
      if ( bullet[bltIdx].x == BulletImpl.NOT_EXIST ) {
        return bullet[bltIdx];
      }
    }
    return null;
  }

  public ActionImpl getActionImplInstance() {
    for ( int i=ACTION_MAX-1 ; i>=0 ; i-- ) {
      actIdx++; actIdx &= (ACTION_MAX-1);
      if ( action[actIdx].pc == ActionImpl.NOT_EXIST ) {
        return action[actIdx];
      }
    }
    return null;
  }

  private int bulletNum;

  public void moveBullets() {
    bulletNum = 0;
    for ( int i=BULLET_MAX-1 ; i>=0 ; i-- ) {
      if ( bullet[i].x != BulletImpl.NOT_EXIST ) {
        bullet[i].move();
        bulletNum++;
      }
    }
    cnt++;
  }

  public void drawBullets() {
    for ( int i=BULLET_MAX-1 ; i>=0 ; i-- ) {
      if ( bullet[i].x != BulletImpl.NOT_EXIST ) {
        bullet[i].draw();
      }
    }
  }

  public void drawBullet(int x, int y, int px, int py, int cl) {
    player.drawBullet(x, y, px, py ,cl);
  }

  private BulletImpl topBullet = null;
  private IActionElmChoice[] topActions;
  private int topx, topy, topd;
  private float toprank;

  public void addTopBullet(IActionElmChoice[] actions,
   int x, int y, int d, float rank) {
    topActions = actions;
    topx = x; topy = y; topd = d; toprank = rank;
    topBullet = getBulletImplInstance();
    topBullet.set(actions, x, y, d, 0, rank, true);
  }

  public boolean isFinished() {
    if ( cnt > CNT_MAX ) return true;
    return ( cnt > 1 && bulletNum <= 0 );
  }

  public int getBulletsNum() {
    if ( cnt > CNT_MAX ) return 0;
    return bulletNum;
  }

  public void rewind() {
    topBullet = getBulletImplInstance();
    topBullet.set(topActions, topx, topy, topd, 0, toprank, true);
    cnt = 0;
  }
}
