/*
 * $Id: BulletmlPlayer.java,v 1.1.1.1 2002/07/07 10:27:57 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.bulletsmorph;

import jp.gr.java_conf.abagames.bulletml.*;
import jp.gr.java_conf.abagames.util.*;
import java.io.*;
import java.util.Vector;
import java.util.Random;
import java.awt.TextArea;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;

/**
 * Handle bullets player.
 *
 * @version $Revision: 1.1.1.1 $
 */
public class BulletmlPlayer implements ITime {
  private Bulletsmorph morph;

  private BulletmlManager manager;
  private Screen screen;

  public void init(BulletmlManager manager, Screen screen, Bulletsmorph morph) {
    this.manager = manager;
    manager.setPlayer(this);
    this.screen = screen;
    this.morph = morph;
  }

  private final int bulletColor[] = {
    0xff006666, 0xff660066, 0xff555500
  };

  public final void drawBullet(int x, int y, int px, int py, int cl) {
    screen.drawLine(x>>8, y>>8, px>>8, py>>8, bulletColor[cl%3]);
  }

  private Timer timer;
  private final int INTERVAL = 16;

  public void start() {
    timer = new Timer(this);
    timer.activate(INTERVAL, false);
  }

  public void restart() {
    if ( timer != null ) {
      timer.deactivate();
    }
  }

  public void terminate() {
    timer.terminate();
  }

  private synchronized void waitFrame() {
    try {
      wait();
    } catch ( InterruptedException e ) {}
  }

  public synchronized final void notifyFrame() {
    notifyAll();
  }

  public final void wakeUp() {
    loop();
    waitFrame();
  }

  private int sx, sy;

  public void setShipPos(int x, int y) {
    sx = x; sy = y;
  }

  private long prvTickCount = 0;

  private void loop() {
    screen.clear();
    long nowTick = System.currentTimeMillis();
    int frame = (int)(nowTick-prvTickCount) / INTERVAL;
    if ( frame <= 0 ) {
      frame = 1;
      prvTickCount = nowTick;
    } else if ( frame > 5 ) {
      frame = 5;
      prvTickCount = nowTick;
    } else {
      prvTickCount += frame*INTERVAL;
    }
    for ( int i=0 ; i<frame ; i++ ) {
      manager.moveBullets();
    }
    manager.drawBullets();

    screen.drawLine(sx, sy-8, sx-6, sy+4, 0xff000000);
    screen.drawLine(sx-6, sy+4, sx+6, sy+4, 0xff000000);
    screen.drawLine(sx+6, sy+4, sx, sy-8, 0xff000000);

    screen.swap();
    if ( manager.isFinished() ) {
      morph.generationFinished();
    }
  }
}
